unit UGraph;

interface

uses
  Controls, Contnrs, Classes, ExtCtrls, StdCtrls, Forms, Graphics;

type
    TGraphTimeUnits = (gtuHours,
                       gtuDays);
    TZoomFactor = 1..10;                       

type
  CGraph = class(TWinControl)
  private
    FpbLeft : TPaintBox;
    FpbMain : TPaintBox;
    FtmpLabel : TLabel;
    FsbMain : TScrollBar;

    FDataList : TObjectList;
    FDTStart : TDateTime;
    FDTEnd : TDateTime;
    FTopValue : Integer;
    FTimeUnits : TGraphTimeUnits;
    FZoomFactor : TZoomFactor;
    FForeColor : TColor;
    FBackColor : TColor;

    procedure MakeGraphOutlinesAndBottomLegend;
    procedure MakeGraphBackGround;
    procedure MakeLeftLegend;
    procedure DrawData;
    procedure pbMainPaint(Sender: TObject);
    procedure sbMainChange(Sender: TObject);
    procedure pbLeftPaint(Sender: TObject);
    procedure SetZoomFactor(aZoomFactor : TZoomFactor);
    function GetDTScrollPos : TDateTime;
    procedure SetDTScrollPos(aDTScrollPos : TDateTime);
    function GetNumPixelsPerMinute : Real;
    procedure SetDTStart(aDTStart : TDateTime);
    procedure SetDTEnd(aDTEnd : TDateTime);
    procedure SetTimeUnits(aTimeUnits : TGraphTimeUnits);
    procedure SetScrollBarWidth;
  public
    constructor create(aOwner : TComponent); override;
    procedure AddData(aDTStart, aDTEnd : TDateTime; aValue : Integer);
    procedure ClearData;
    procedure Repaint; override;
    procedure NormalizeData;

    property ZoomFactor : TZoomFactor read FZoomFactor write SetZoomFactor;
    property DTScrollPos : TDateTime read GetDTScrollPos write SetDTScrollPos;
  published
    property Align;
    property Anchors;
    property Constraints;
    property Enabled;
    property PopupMenu;
    property ShowHint;
    property Visible;
    property OnEnter;
    property OnExit;
    property Color;
    property DTStart : TDateTime read FDTStart write SetDTStart;
    property DTEnd : TDateTime read FDTEnd write SetDTEnd;
    property TopValue : Integer read FTopValue write FTopValue;
    property TimeUnits : TGraphTimeUnits read FTimeUnits write SetTimeUnits;
    property ForeColor : TColor read FForeColor write FForeColor;
    property BackColor : TColor read FBackColor write FBackColor;
  end;

function RuleListSortFunc(Item1, Item2: Pointer): Integer;

procedure Register;

implementation

uses
  Math, SysUtils, Windows, DateUtils, UGraphUtils;

type
  CData = class
  public
    DTStart : TDateTime;
    DTEnd : TDateTime;
    Value : Integer;
    constructor Create(aDTStart, aDTEnd : TDateTime; aValue : Integer);
  end;

  CRule = class
  public
    DateTime : TDateTime;
    Value : Integer;
    constructor Create(aDateTime : TDateTime; aValue : Integer);
  end;

const
    BOTTOM_SPACE_HEIGHT = 51;
    TOP_SPACE_HEIGHT = 10;
    LEFT_PANEL_WIDTH = 93;
    NUM_VALUE_INTERVALS = 11;
    SPACE_TO_RIGHT_OF_VALUE_LABEL = 11;
    DEFAULT_WIDTH_BTWN_TICKS = 60;
    TEXT_HEIGHT_OFFSET = -5;

procedure Register;
begin
  RegisterComponents('NDSUtils', [CGraph]);
end;

constructor CData.Create(aDTStart, aDTEnd : TDateTime; aValue : Integer);
begin
    DTStart := aDTStart;
    DTEnd := aDTEnd;
    Value := aValue;
end;

constructor CRule.Create(aDateTime : TDateTime; aValue : Integer);
begin
    DateTime := aDateTime;
    Value := aValue;
end;

constructor CGraph.Create(aOwner : TComponent);
var
    aPanel : TPanel;
begin
    inherited Create(aOwner);
    FDataList := TObjectList.Create;
    FDTStart := DBN_Now;
    FDTEnd := DT_IncDays(DBN_Now, 7);
    FTopValue := 10000;
    FTimeUnits := gtuHours;
    FZoomFactor := 1;
    FForeColor := clPurple;
    FBackColor := clWhite;

    Height := 353;
    Width := 688;
    Constraints.MinWidth := 100;

    FpbLeft := TPaintBox.Create(Self);
    with FpbLeft do begin
        Parent := Self;
        Width := LEFT_PANEL_WIDTH;
        Align := alLeft;
        Enabled := False;
        OnPaint := pbLeftPaint;
    end;

    FtmpLabel := TLabel.Create(Self);
    with FtmpLabel do begin
        Visible := false;
    end;

    aPanel := TPanel.Create(Self);
    with aPanel do begin
        Parent := Self;
        Align := alClient;
        BevelInner := bvNone;
        BevelOuter := bvNone;
        BorderStyle := bsNone;
    end;

    FsbMain := TScrollBar.Create(Self);
    with FsbMain do begin
        Parent := aPanel;
        Kind := sbHorizontal;
        Align := alBottom;
        Min := 0;
        OnChange := sbMainChange;
        SmallChange := 1;
        LargeChange := 12;
    end;

    FpbMain := TPaintBox.Create(Self);
    with FpbMain do begin
        Parent := aPanel;
        Align := alClient;
        Enabled := False;
        OnPaint := pbMainPaint;
        DoubleBuffered := true;
    end;
end;

procedure CGraph.MakeLeftLegend;
var
    i, aWidth, aYPos, aValue : Integer;
begin
    FpbLeft.Canvas.Pen.Color := clBlack;
    for i := 0 to NUM_VALUE_INTERVALS-1 do begin
        aValue := FTopValue - Ceil(1.0*i*FTopValue/(NUM_VALUE_INTERVALS-1));
        FtmpLabel.Caption := STR_IntToDigitGroupedString(aValue);
        aWidth := FtmpLabel.Width;
        aYPos := TOP_SPACE_HEIGHT + Ceil(1.0*i*(Height - BOTTOM_SPACE_HEIGHT - TOP_SPACE_HEIGHT)/(NUM_VALUE_INTERVALS-1));
        FpbLeft.Canvas.TextOut(LEFT_PANEL_WIDTH - SPACE_TO_RIGHT_OF_VALUE_LABEL - aWidth, aYPos + TEXT_HEIGHT_OFFSET, STR_IntToDigitGroupedString(aValue));
        FpbLeft.Canvas.MoveTo(LEFT_PANEL_WIDTH - 5, aYPos);
        FpbLeft.Canvas.LineTo(LEFT_PANEL_WIDTH - 1, aYPos);
    end;
    FpbLeft.Canvas.MoveTo(LEFT_PANEL_WIDTH - 1, TOP_SPACE_HEIGHT);
    FpbLeft.Canvas.LineTo(LEFT_PANEL_WIDTH - 1, Height - BOTTOM_SPACE_HEIGHT + 1);
end;

procedure CGraph.MakeGraphBackGround;
begin
    FpbMain.Canvas.Brush.Color := FBackColor;
    FpbMain.Canvas.Rectangle(0, TOP_SPACE_HEIGHT-1, FpbMain.Width, Height - BOTTOM_SPACE_HEIGHT+1);
    FpbMain.Canvas.Brush.Color := Color;
end;

procedure CGraph.MakeGraphOutlinesAndBottomLegend;
var
    i, aYPos, aXPos, AvgBackColorHue : Integer;
    aNumPixelsPerMinute : Real;
    aDateTime, aDTScrollPos : TDateTime;
begin
    if (FDTEnd = FDTStart) then
        Exit;
    FpbMain.Canvas.Pen.Style := psDot;
    AvgBackColorHue := (GetRValue(FBackColor) + GetGValue(FBackColor) + GetBValue(FBackColor)) div 3;
    if (AvgBackColorHue < 127) then
        FpbMain.Canvas.Pen.Color := clWhite
    else
        FpbMain.Canvas.Pen.Color := clBlack;
    for i := 1 to NUM_VALUE_INTERVALS-2 do begin
        aYPos := TOP_SPACE_HEIGHT + Ceil(1.0*i*(Height - BOTTOM_SPACE_HEIGHT - TOP_SPACE_HEIGHT)/(NUM_VALUE_INTERVALS-1));
        FpbMain.Canvas.MoveTo(0, aYPos);
        FpbMain.Canvas.LineTo(FpbMain.Width - 1, aYPos);
    end;
    FpbMain.Canvas.Pen.Color := clBlack;
    FpbMain.Canvas.Pen.Style := psSolid;
    aNumPixelsPerMinute := GetNumPixelsPerMinute;
    aXPos := 0;
    aDTScrollPos := DTScrollPos;
    while true do begin
        aXPos := aXPos+(DEFAULT_WIDTH_BTWN_TICKS*FZoomFactor);
        if (aXPos > FpbMain.Width) then
            Break;

        aDateTime := DT_IncMinutes(aDTScrollPos, Ceil(aXPos/aNumPixelsPerMinute));

        if (AvgBackColorHue < 127) then
            FpbMain.Canvas.Pen.Color := clWhite
        else
            FpbMain.Canvas.Pen.Color := clBlack;
        FpbMain.Canvas.Pen.Style := psDot;
        FpbMain.Canvas.MoveTo(aXPos, TOP_SPACE_HEIGHT);
        FpbMain.Canvas.LineTo(aXPos, Height - BOTTOM_SPACE_HEIGHT);
        FpbMain.Canvas.Pen.Color := clBlack;
        FpbMain.Canvas.Pen.Style := psSolid;

        FpbMain.Canvas.MoveTo(aXPos, Height - BOTTOM_SPACE_HEIGHT);
        FpbMain.Canvas.LineTo(aXPos, Height - BOTTOM_SPACE_HEIGHT + 4);
        FpbMain.Canvas.TextOut(aXPos - 25, Height - BOTTOM_SPACE_HEIGHT + 9 + TEXT_HEIGHT_OFFSET,
                               FormatDateTime('d/m/yy', aDateTime));
        if (FTimeUnits = gtuHours) then
            FpbMain.Canvas.TextOut(aXPos - 16, Height - BOTTOM_SPACE_HEIGHT + 23 + TEXT_HEIGHT_OFFSET,
                                   FormatDateTime('hh:nn', aDateTime));
    end;
end;

procedure CGraph.DrawData;
var
    i, aStartXPos, aEndXPos : Integer;
    aData : CData;
    aNumPixelsPerMinute, aNumPixelsPerValueUnit : Real;
    aDTScrollPos : TDateTime;
begin
    if (FTopValue = 0) then
        Exit;
    aNumPixelsPerMinute := GetNumPixelsPerMinute;
    aNumPixelsPerValueUnit := (Height - BOTTOM_SPACE_HEIGHT - TOP_SPACE_HEIGHT)/FTopValue;
    aDTScrollPos := DTScrollPos;
    FpbMain.Canvas.Pen.Color := FForeColor;
    FpbMain.Canvas.Brush.Color := FForeColor;
    aEndXPos := 0;
    for i := 0 to FDataList.Count-1 do begin
        aData := FDataList.Items[i] as CData;
        if (aData.DTEnd < aDTScrollPos) then
            Continue
        else if (aData.DTStart > DT_IncMinutes(aDTScrollPos, Ceil(FpbMain.Width/aNumPixelsPerMinute))) then
            Break;
        aStartXPos := Round(DT_DateTimeDiffInMinutes(aData.DTStart, aDTScrollPos)*aNumPixelsPerMinute);
        if (aStartXPos < aEndXPos) then
            inc(aStartXPos);
        if (aStartXPos < 0) then
            aStartXPos := 0;
        aEndXPos := Round(DT_DateTimeDiffInMinutes(aData.DTEnd, aDTScrollPos)*aNumPixelsPerMinute);
        if (aEndXPos > FpbMain.Width-1) then
            aEndXPos := FpbMain.Width-1;
        if (aEndXPos - aStartXPos < 1) then
            inc(aEndXPos);            
        FpbMain.Canvas.Rectangle(aStartXPos, Height - BOTTOM_SPACE_HEIGHT - Ceil(aData.Value*aNumPixelsPerValueUnit),
                                 aEndXPos, Height - BOTTOM_SPACE_HEIGHT);
    end;
    FpbMain.Canvas.Brush.Color := clBtnFace;
    FpbMain.Canvas.Pen.Color := clBlack;
end;

function CGraph.GetNumPixelsPerMinute : Real;
begin
    Result := DEFAULT_WIDTH_BTWN_TICKS*FZoomFactor;
    case FTimeUnits of
        gtuHours : Result := Result/DT_ONE_HOUR_IN_MINUTES;
        gtuDays : Result := Result/DT_ONE_DAY_IN_MINUTES;
    end;
end;

function CGraph.GetDTScrollPos : TDateTime;
begin
    Result := 0;
    case FTimeUnits of
        gtuHours : Result := DT_IncHours(FDTStart, FsbMain.Position);
        gtuDays : Result := DT_IncDays(FDTStart, FsbMain.Position);
    end;
end;

procedure CGraph.SetDTScrollPos(aDTScrollPos : TDateTime);
begin
    if ((aDTScrollPos < FDTStart) and (aDTScrollPos > FDTEnd)) then
        Exit;
    case FTimeUnits of
        gtuHours : FsbMain.Position := Ceil(DT_DateTimeDiffInHours(aDTScrollPos, FDTStart));
        gtuDays : FsbMain.Position := Ceil(aDTScrollPos - FDTStart);
    end;
end;

procedure CGraph.pbLeftPaint(Sender: TObject);
begin
    MakeLeftLegend;
end;

procedure CGraph.AddData(aDTStart, aDTEnd : TDateTime; aValue : Integer);
begin
    FDataList.Add(CData.Create(aDTStart, aDTEnd, aValue));
end;

procedure CGraph.NormalizeData;
var
    aRuleList : TObjectList;
    i, aValue, aOldValue : Integer;
    aRule1, aRule2 : CRule;
    aDTStart, aDTEnd, aOldDTStart, aOldDTEnd : TDateTime;
    aData : CData;
begin
    aRuleList := TObjectList.Create;
    try
        aOldDTStart := 0;
        aOldDTEnd := 0;
        aOldValue := 0;
        for i := 0 to FDataList.Count-1 do begin
            aData := FDataList.Items[i] as CData;
            if (aData.Value = 0) then
                Continue;
            if ((aOldDTEnd = aData.DTStart) and (aOldValue = aData.Value)) then
                aOldDTEnd := aData.DTEnd
            else begin
                if (aOldValue <> 0) then begin
                    aRuleList.Add(CRule.create(aOldDTStart, aOldValue));
                    aRuleList.Add(CRule.create(aOldDTEnd, -aOldValue));
                end;
                aOldDTStart := aData.DTStart;
                aOldDTEnd := aData.DTEnd;
                aOldValue := aData.Value;
            end;
        end;
        aRuleList.Add(CRule.create(aOldDTStart, aOldValue));
        aRuleList.Add(CRule.create(aOldDTEnd, -aOldValue));

        ClearData;
        aRuleList.Sort(RuleListSortFunc);
        aValue := 0;
        for i := 0 to aRuleList.Count-2 do begin
            aRule1 := aRuleList.Items[i] as CRule;
            aRule2 := aRuleList.Items[i+1] as CRule;
            aValue := aValue + aRule1.Value;
            if (aValue = 0) then
                Continue;
            aDTStart := aRule1.DateTime;
            aDTEnd := aRule2.DateTime;
            if (aDTStart <> aDTEnd) then
                AddData(aDTStart, aDTEnd, aValue);
        end;
    finally
        aRuleList.Free;
    end;
end;


procedure CGraph.pbMainPaint(Sender: TObject);
begin
    MakeGraphBackGround;
    DrawData;
    MakeGraphOutlinesAndBottomLegend;
end;

procedure CGraph.sbMainChange(Sender: TObject);
begin
    pbMainPaint(Self);
end;

procedure CGraph.ClearData;
begin
    FDataList.Clear;
end;

procedure CGraph.Repaint;
begin
    FpbLeft.Repaint;
    FpbMain.Repaint;
    FsbMain.Repaint;
end;

procedure CGraph.SetZoomFactor(aZoomFactor : TZoomFactor);
begin
    if ((aZoomFactor < low(TZoomFactor)) or (aZoomFactor > high(TZoomFactor))) then
        Raise ERangeError.Create('ZoomFactor out of range');
    FZoomFactor := aZoomFactor;
    FpbMain.Repaint;
    FsbMain.Repaint;
end;

procedure CGraph.SetDTStart(aDTStart : TDateTime);
var
    aHour : Word; 
begin
    case FTimeUnits of
        gtuHours : begin
            aHour := HourOf(FDTStart);
            FDTStart := DT_IncHours(Floor(aDTStart), aHour);
        end;
        gtuDays : FDTStart := Floor(aDTStart);
    end;        
    SetScrollBarWidth;
end;

procedure CGraph.SetDTEnd(aDTEnd : TDateTime);
begin
    FDTEnd := aDTEnd;
    SetScrollBarWidth;
end;

procedure CGraph.SetTimeUnits(aTimeUnits : TGraphTimeUnits);
begin
    FTimeUnits := aTimeUnits;
    DTStart := FDTStart;
end;

procedure CGraph.SetScrollBarWidth;
var
    aNumUnits : Integer;
begin
    if (FDTEnd = FDTStart) then
        Exit;
    aNumUnits := 0;
    case FTimeUnits of
        gtuHours : aNumUnits := DT_DateTimeDiffInHours(FDTEnd, FDTStart);
        gtuDays : aNumUnits := Ceil(FDTEnd - FDTStart);
    end;
    FsbMain.Max := aNumUnits;
end;

function RuleListSortFunc(Item1, Item2: Pointer): Integer;
var
    aRule1, aRule2 : CRule;
begin
    aRule1 := CRule(Item1);
    aRule2 := CRule(Item2);
    if (aRule1.DateTime < aRule2.DateTime) then
        Result := -1
    else if (aRule1.DateTime > aRule2.DateTime) then
        Result := 1
    else
        Result := 0;
end;

end.
